/**
 * @file main.c
 * @author Florian Burgener (florian.burgener@etu.hesge.ch)
 * @brief Démonstration de libraire stack avec l'implémentation du tri à deux piles.
 * @version 1.0
 * @date 2021-11-23
 *
 * @copyright Copyright (c) 2021
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "stack.h"

typedef enum _sort_behavior {
    behavior_lower,
    behavior_higher,
    behavior_none,
} sort_behavior;

void pop_left_push_right(stack *left_stack, stack *right_stack, int current_value, sort_behavior sb) {
    while (!stack_is_empty(*left_stack)) {
        int tmp;
        stack_peek(*left_stack, &tmp);

        if (sb == behavior_lower) {
            if (current_value < tmp) {
                break;
            }
        } else if (sb == behavior_higher) {
            if (current_value > tmp) {
                break;
            }
        }

        stack_pop(left_stack, &tmp);
        stack_push(right_stack, tmp);
    }
}

void sort_array(int *array, int array_length) {
    stack left_stack;
    stack_init(&left_stack, array_length);
    stack right_stack;
    stack_init(&right_stack, array_length);

    for (int i = 0; i < array_length; i += 1) {
        // Pop values from left stack to right stack.
        pop_left_push_right(&left_stack, &right_stack, array[i], behavior_lower);
        // Pop values from right stack to left stack.
        pop_left_push_right(&right_stack, &left_stack, array[i], behavior_higher);
        // Push to left stack.
        stack_push(&left_stack, array[i]);
    }

    // Until right stack is empty pop value from right stack and push it to left stack.
    pop_left_push_right(&right_stack, &left_stack, 0, behavior_none);

    // Until the left stack is empty, pop the value and place it in the array.
    for (int i = 0; i < array_length; i += 1) {
        int tmp;
        stack_pop(&left_stack, &tmp);
        array[i] = tmp;
    }

    stack_destroy(&left_stack);
    stack_destroy(&right_stack);
}

void print_array(int *array, int array_length) {
    for (int i = 0; i < array_length; i += 1) {
        printf("%d, ", array[i]);
    }
    printf("\n");
}

int main(int argc, char **argv) {
    if (argc < 2) {
        printf("Too few arguments.\n");
        return EXIT_FAILURE;
    }

    int array_length = atoi(argv[1]);

    if (array_length != argc - 2 || array_length == 0) {
        printf("The length of the array does not correspond to the number of elements.\n");
        return EXIT_FAILURE;
    }

    int *array = (int *)malloc(sizeof(int) * array_length);

    for (int i = 0; i < argc - 2; i += 1) {
        array[i] = atoi(argv[i + 2]);
    }

    print_array(array, array_length);
    sort_array(array, array_length);
    print_array(array, array_length);

    free(array);
    return EXIT_SUCCESS;
}
