/**
 * @file stack.c
 * @author Florian Burgener (florian.burgener@etu.hesge.ch)
 * @brief Implémentation de librairie stack.
 * @version 1.0
 * @date 2021-11-23
 *
 * @copyright Copyright (c) 2021
 *
 */

#include "stack.h"

#include <stdbool.h>
#include <stdlib.h>

void stack_init(stack *s, int size) {
    s->size = size;
    s->top = -1;
    s->data = (int *)malloc(sizeof(int) * s->size);
}

bool stack_is_empty(stack s) {
    return s.top == -1;
}

void stack_push(stack *s, int val) {
    if (s->size == s->top + 1) {
        return;
    }

    s->top += 1;
    s->data[s->top] = val;
}

void stack_pop(stack *s, int *val) {
    if (stack_is_empty(*s)) {
        return;
    }

    stack_peek(*s, val);
    s->top -= 1;
}

void stack_peek(stack s, int *val) {
    if (stack_is_empty(s)) {
        return;
    }

    *val = s.data[s.top];
}

void stack_destroy(stack *s) {
    s->size = -1;
    s->top = -1;
    free(s->data);
    s->data = NULL;
}
