/**
 * @file stack_tests.c
 * @author Florian Burgener (florian.burgener@etu.hesge.ch) et Quentin Fasler
 * @brief Seul les tests ont été fait en commun pour gagner du temps.
 * @version 1.0
 * @date 2021-11-23
 *
 * @copyright Copyright (c) 2021
 *
 */

#include <assert.h>
#include <stdlib.h>

#include "stack.h"

int main() {
    const int STACK_LENGTH = 10;

    // Test : stack_init
    stack s1;
    stack_init(&s1, STACK_LENGTH);
    assert(s1.size == 10);
    assert(s1.top == -1);
    assert(s1.data != NULL);

    // Test : stack_destroy
    stack_destroy(&s1);
    assert(s1.size == -1);
    assert(s1.top == -1);
    assert(s1.data == NULL);

    // Test : stack_push
    stack s2;
    stack_init(&s2, STACK_LENGTH);
    stack_push(&s2, 1);
    assert(s2.data[0] == 1);
    assert(s2.top == 0);
    stack_push(&s2, 10);
    assert(s2.data[1] == 10);
    assert(s2.top == 1);
    stack_push(&s2, 100);
    assert(s2.data[2] == 100);
    assert(s2.top == 2);
    stack_destroy(&s2);

    // Test : stack_push (size overflow)
    stack s2b;
    stack_init(&s2b, 2);
    stack_push(&s2b, 1);
    stack_push(&s2b, 2);
    // Overflow from here.
    stack_push(&s2b, 3);
    stack_push(&s2b, 4);
    assert(s2b.top == 1);
    stack_destroy(&s2b);

    // Test : stack_is_empty
    stack s3;
    stack_init(&s3, STACK_LENGTH);
    assert(stack_is_empty(s3));
    stack_push(&s3, 1);
    assert(!stack_is_empty(s3));
    stack_destroy(&s3);

    // Test : stack_pop
    stack s4;
    stack_init(&s4, STACK_LENGTH);
    stack_push(&s4, 1);
    stack_push(&s4, 10);
    int val1;
    stack_pop(&s4, &val1);
    assert(val1 == 10);
    assert(s4.top == 0);
    stack_pop(&s4, &val1);
    assert(val1 == 1);
    assert(s4.top == -1);
    stack_destroy(&s4);

    // Test : stack_pop (stack is empty)
    stack s4b;
    stack_init(&s4b, STACK_LENGTH);
    int val1b;
    stack_pop(&s4b, &val1b);
    assert(s4b.top == -1);
    stack_destroy(&s4b);

    // Test : stack_peek
    stack s5;
    stack_init(&s5, STACK_LENGTH);
    stack_push(&s5, 1);
    int val2;
    stack_peek(s5, &val2);
    assert(val2 == 1);
    assert(s5.top == 0);
    stack_destroy(&s5);

    // Test : stack_peek (stack is empty)
    stack s5b;
    stack_init(&s5b, STACK_LENGTH);
    int val2b;
    stack_peek(s5b, &val2b);
    assert(s5b.top == -1);
    stack_destroy(&s5b);
}
